@@ -1,6 +1,6 @@
 require 'spec_helper'
 
-describe Sinatra::Namespace do
+RSpec.describe Sinatra::Namespace do
   verbs = [:get, :head, :post, :put, :delete, :options, :patch]
 
   def mock_app(&block)
@@ -263,6 +263,7 @@ describe Sinatra::Namespace do
 
           specify 'are accepted in the before-filter' do
             namespace '/foo' do
+              before { @yes = nil }
               before(:host_name => 'example.com') { @yes = 'yes' }
               send(verb) { @yes || 'no' }
             end
@@ -661,10 +662,10 @@ describe Sinatra::Namespace do
 
           specify 'can use a custom views directory' do
             mock_app do
-              set :views, File.expand_path('../namespace', __FILE__)
+              set :views, File.expand_path('namespace', __dir__)
               send(verb, '/') { erb :foo }
               namespace('/foo') do
-                set :views, File.expand_path('../namespace/nested', __FILE__)
+                set :views, File.expand_path('namespace/nested', __dir__)
                 send(verb) { erb :foo }
               end
             end
@@ -702,6 +703,28 @@ describe Sinatra::Namespace do
             expect(send(verb, '/').body).to eq 'Hello World!'
             expect(send(verb, '/foo').body).to eq 'Hi World!'
           end
+
+          specify 'can render strings' do
+            mock_app do
+              namespace '/foo' do
+                send(verb) { erb 'foo' }
+              end
+            end
+
+            expect(send(verb, '/foo').body).to eq 'foo'
+          end
+
+          specify 'can render strings nested' do
+            mock_app do
+              namespace '/foo' do
+                namespace '/bar' do
+                  send(verb) { erb 'bar' }
+                end
+              end
+            end
+
+            expect(send(verb, '/foo/bar').body).to eq 'bar'
+          end
         end
       end
 
@@ -773,9 +796,9 @@ describe Sinatra::Namespace do
     it 'sets hashes correctly' do
       mock_app do
         namespace '/foo' do
-          set erb: 'o', sass: 'k'
+          set erb: 'o', haml: 'k'
           get '/bar' do
-            settings.erb + settings.sass
+            settings.erb + settings.haml
           end
         end
       end
